#[cfg(not(target_env = "msvc"))]
use tikv_jemallocator::Jemalloc;

#[cfg(not(target_env = "msvc"))]
#[global_allocator]
static GLOBAL: Jemalloc = Jemalloc;

use actix_web::{get, http, middleware, web, App, HttpResponse, HttpServer};
use actix_web_static_files::ResourceFiles;
use r2d2_redis::redis;
use std::ops::DerefMut;

use bvplan::*;

pub mod static_dir {
    include!(concat!(env!("OUT_DIR"), "/generated.rs"));
}

async fn not_found() -> HttpResponse {
    templates::StatusCode {
        status_code: http::StatusCode::NOT_FOUND,
        message: None,
    }
    .to_response()
}

#[get("/")]
async fn index(redis_pool: web::Data<cache::RedisPool>) -> HttpResponse {
    let redis_conn = &mut match redis_pool.get() {
        Ok(x) => x,
        Err(_) => {
            return templates::StatusCode {
                status_code: http::StatusCode::INTERNAL_SERVER_ERROR,
                message: None,
            }
            .to_response()
        }
    };

    if let Some(html) = match redis::cmd("GET")
        .arg(cache::keys::SUBSTITUTIONS_HTML)
        .query::<Option<String>>(redis_conn.deref_mut())
    {
        Ok(x) => x,
        Err(_) => {
            return templates::StatusCode {
                status_code: http::StatusCode::INTERNAL_SERVER_ERROR,
                message: None,
            }
            .to_response()
        }
    } {
        HttpResponse::Accepted()
            .content_type("text/html; charset=utf-8")
            .body(html)
    } else {
        templates::StatusCode {
            status_code: http::StatusCode::INTERNAL_SERVER_ERROR,
            message: None,
        }
        .to_response()
    }
}

#[actix_web::main]
async fn main() -> std::io::Result<()> {
    env_logger::init();

    HttpServer::new(move || {
        let generated = static_dir::generate();

        App::new()
            .app_data(web::Data::new(cache::pool().unwrap()))
            .wrap(middleware::Compress::default())
            .wrap(middleware::Logger::default())
            .service(index)
            .service(ResourceFiles::new("/static", generated))
            .default_service(web::route().to(not_found))
    })
    .bind("0.0.0.0:80")
    .unwrap()
    .run()
    .await
}
