use actix_web::{body::BoxBody, http, HttpResponse, HttpResponseBuilder, ResponseError};
use askama::Template;
use std::fmt;

use crate::cache;

struct ActixError(askama::Error);

impl fmt::Debug for ActixError {
    #[inline]
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        <askama::Error as fmt::Debug>::fmt(&self.0, f)
    }
}

impl fmt::Display for ActixError {
    #[inline]
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        <askama::Error as fmt::Display>::fmt(&self.0, f)
    }
}

impl ResponseError for ActixError {}

pub trait TemplateToResponseWithStatusCode {
    fn to_response_with_status_code(&self, code: http::StatusCode) -> HttpResponse<BoxBody>;
}

impl<T: askama_actix::Template> TemplateToResponseWithStatusCode for T {
    fn to_response_with_status_code(&self, code: http::StatusCode) -> HttpResponse<BoxBody> {
        match self.render() {
            Ok(buffer) => HttpResponseBuilder::new(code)
                .content_type(http::header::HeaderValue::from_static(T::MIME_TYPE))
                .body(buffer),
            Err(err) => HttpResponse::from_error(ActixError(err)),
        }
    }
}

#[derive(Template)]
#[template(path = "bvplan.html")]
pub struct BVplan {
    pub data: cache::keys::substitutions::SubstitutionQuery,
}

#[derive(Template)]
#[template(path = "status_code.html")]
pub struct StatusCode {
    pub status_code: http::StatusCode,
    pub message: Option<String>,
}

impl StatusCode {
    pub fn to_response(&self) -> HttpResponse<BoxBody> {
        self.to_response_with_status_code(self.status_code)
    }
}
